#include <stdio.h>
#include "iksemel.h"

void packetRecv(void *, ikspak *);
void usage(char *prog);

typedef struct my_data {
  char * server;
  char * username;
  char * password;
  char * resource;
  char * dest;
  char *message;
  iksparser *parser;
} my_data;

int main(int argc, char *argv[])
{
  if (argc != 7) {
          usage(argv[0]);
  }
  my_data *data = (my_data *)malloc(sizeof(my_data));
  data->server = argv[1];
  data->username = argv[2];
  data->password = argv[3];
  data->resource = argv[4];
  data->dest = argv[5];
  data->message = argv[6];
  printf("%s@%s/%s Sending Message to: %s\n", 
                  data->username, data->server, data->resource, data->dest);

  data->parser = iks_jabber_new(data, packetRecv);

  if (!iks_connect_tcp(data->parser, data->server, 0)) {
          printf("Error connecting to %s\n",data->server);
          exit(1);
  }

  while(iks_recv(data->parser, 3000));
  exit(0);
}
        

void usage(char *prog)
{
  printf("USAGE: %s <server> <username> ", prog);
  printf("<password> <resource> <destjid> <message>\n");
  exit(1);
}

void packetRecv(void *d, ikspak *pkt)
{
  my_data *data = (my_data *)d;
  if (pkt->type == IKS_PAK_STREAM) {
    iks* x;
    char *jid = (char *)malloc(1024);
    sprintf(jid, "%s@%s/%s", data->username, data->server, data->resource);
    iksid *id = iks_id_new(NULL, jid);

    x = iks_make_auth(id, data->password, iks_find_attrib(pkt->x, "id"));
    iks_insert_attrib(x, "id", "auth");
    iks_send(data->parser, x);
    iks_delete(x);
    free(jid);
  }
  if (pkt->type == IKS_PAK_IQ) {
    if (pkt->subtype == IKS_TYPE_ERROR) {
        printf("ERROR Authenticating\n");
        exit(1);
    }
    iks *x = iks_make_msg(IKS_TYPE_HEADLINE, data->dest, 
		    data->message, "thread-id");
    iks_insert_cdata(iks_insert(x, "subject"), 
		    "A Message From Me", -1); // -1 for len means use strlen()
    iks_send(data->parser, x);
    iks_delete(x);

    iks_disconnect(data->parser);
  }
  
}

