require 'tk'
require 'jabber4r/jabber4r'


def make_gui
  # Create a root window with three partitions in it
  root = TkRoot.new {title "Jabber Chat"}
  button_frame = TkFrame.new(root).pack("side"=>"bottom")
  bframe = TkFrame.new(root).pack("side"=>"bottom")
  tframe = TkFrame.new(root).pack("side"=>"top")

  # The bottom frame gets a button and three text boxes
  lframe = TkFrame.new(button_frame).pack("side"=>"left")
  lab = TkLabel.new(lframe) {text "Your  JID"}.pack("side"=>"left")
  $jid_entry = TkEntry.new(lframe).pack("side"=>"right")
  $jid_entry.insert("end", "user1@my-jabber/ad")

  lframe = TkFrame.new(button_frame).pack("side"=>"left")
  lab = TkLabel.new(lframe) {text "Password "}.pack("side"=>"left")
  $pass_entry = TkEntry.new(lframe).pack("side"=>"right")
  $pass_entry.insert("end", "user1")

  lframe = TkFrame.new(button_frame).pack("side"=>"left")
  lab = TkLabel.new(lframe) {text "Other JID"}.pack("side"=>"left")
  $chat_entry = TkEntry.new(lframe).pack("side"=>"right")
  $chat_entry.insert("end", "user2@my-jabber")

  $but = TkButton.new(button_frame){text "Connect"}.pack("side"=>"right")
  $but.command() {buttonPress}

  # The top frame gets a multi-line scrolled text window to show messagest
  scroll = TkScrollbar.new(tframe){}.pack("fill"=>"y", "side" => "left")
  $output = TkText.new(tframe) \
    {width 80; height 20}.pack("side" => "top","fill" => "both", 
	                           "anchor" => "n", "expand" => "true")
  $output.yscrollbar(scroll)
  $output.tag_configure("mine", "foreground", "red")
  $output.tag_configure("his", "foreground", "blue")


  # The bottom frame gets a single-line text window for message entry
  $input = TkEntry.new(bframe) {width 80}.pack("side" => "bottom",
                                      "fill" => "both", "anchor" => "s")
  $input.bind("Return") {|x| sendMsg(x) }

  Tk.mainloop
  # Doesn't return
end

def buttonPress()
	# A little cheesy. The first time the button is pushed, it 
	# connects.  All subsequent times, it sends the presence.
	if $but.text == "Connect"
		connectToJabber($jid_entry.value(), $pass_entry.value(), $chat_entry.value())
		$but.configure("text", "Send\nPresence")
	else
		sendPresence
  end
end

def connectToJabber(myjid, passwd, chatjid)
	begin
    $session = Jabber::Session.bind_digest(myjid, passwd)
	  $session.announce_initial_presence
	  $dest = chatjid
	  $myjid = myjid
	  $session.add_roster_listener {|event, roster|
		  rosterCB(event, roster)
	  }
	  $session.add_message_listener {|msg|
		  messageCB(msg.body)
	  }
	  $session.set_subscription_handler {|subscription|
		  subscriptionCB(subscription)
	  }
  rescue
		print "Exception: "+$!.backtrace.join("\n")
  end
end

def sendMsg(x)
	text = x.widget.value().strip
	$input.delete("0", "end")
	msg = $session.new_chat_message($dest)
	msg.set_body(text)
	$output.insert("end", "<#{$myjid.split('@')[0]}> #{text}\n", "mine")
	msg.set_thread("chat_demo_thread")
	msg.set_subject("chat_demo_subject")
	msg.send(false)
end


def sendPresence
	presence = Jabber::Protocol::Presence.new(
	        Jabber.gen_random_id, "chat", "Let's Roll")
	$session.connection.send(presence)
end

def subscriptionCB(subscription) 
    case subscription.type
      when :subscribe
        subscription.accept
	      $session.subscribe(subscription.from)
      when :unsubscribe
        subscription.accept
        p = Jabber::Protocol::Presence.new(Jabber.gen_random_id)
        p.type = "unsubscribe"
        p.to = subscription.from
	      $session.connection.send(p)
    end
end

def rosterCB(event,roster) 
	action = case event
	  when Jabber::Roster::ITEM_ADDED
			"item added"
	  when Jabber::Roster::ITEM_DELETED
			"item deleted"
	  when Jabber::Roster::RESOURCE_ADDED
			"resource added"
	  when Jabber::Roster::RESOURCE_UPDATED
			"resource updated"
	  when Jabber::Roster::RESOURCE_DELETED
			"resource deleted"
		end
	text = "<roster #{action}>"
	text << "#{$dest}/#{roster.name} is "
	text << "#{roster.show}:#{roster.status}\n"
	$output.insert("end", text)
end

def messageCB(text) 
	$output.insert("end", "<#{$dest.split('@')[0]}> #{text}\n", "his")
end


make_gui
